#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
Error.stackTraceLimit = Infinity;
const program = require('commander');
const config_1 = require("./commands/config");
const multimerge_1 = require("./commands/multimerge");
const config_file_1 = require("./config-file");
const push_1 = require("./commands/push");
const merge_1 = require("./commands/merge");
const pull_1 = require("./commands/pull");
const status_1 = require("./commands/status");
const packageJson = require('../package.json');
program
    .version(packageJson.version);
program
    .command(`status`)
    .option(`-f, --full`, `Include full hg status (slower)`)
    .description(`show status for all repos`)
    .action(() => execCommand(status_1.status, { full: true || !!program.full })); // hardcode --full to true for now - it may soon be the default
program
    .command(`config`)
    .description(`show hgpp config`)
    .action(() => execCommand(config_1.config));
program
    .command(`pull`)
    .option(`-u, --update`, `Als9o try update to the latest head of the branch`)
    .description(`hg pull on all repos`)
    .action(() => execCommand(pull_1.pull, { tryUpdate: !!program.update }));
program
    .command(`merge`)
    .description(`try merge all repos with latest default`)
    .action(() => execCommand(merge_1.merge));
program
    .command(`push`)
    .description(`hg push on all repos`)
    .action(() => execCommand(push_1.push));
program
    .command(`multimerge <branchFragments...>`)
    .alias(`mm`)
    .description(`merge default into multiple branches. Branches can be specified by fragments (e.g. a case number)`)
    .action((branchFragments) => execCommand(multimerge_1.multimerge, { branchFragments, cwd: process.cwd() }));
program
    .parse(process.argv);
if (!process.argv.slice(2).length) {
    execCommand(status_1.status, { full: !!program.full });
}
function execCommand(command, options = {}) {
    config_file_1.getConfig()
        .then(config => {
        return command(config, options);
    })
        .catch(_ => {
        // TODO better way of doing this than ignoring all errors...
        // Listr task errors will be piped to Listr as expected, but any other uncaught exceptions are gonna be swallowed..
    });
}
//# sourceMappingURL=cli.js.map