"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Listr = require('listr');
const rxjs_1 = require("rxjs");
const config_file_1 = require("../config-file");
const hg_1 = require("../hg");
// TODO refactor this observable goop.
function merge(config) {
    let projTasks = config.projects.map(project => {
        return {
            title: project.path,
            task: (_, projectTask) => {
                return new Listr([
                    {
                        title: 'Pulling...',
                        task: () => hg_1.pullChanges(project.path)
                    },
                    {
                        title: 'Merging...',
                        task: (ctx, task) => tryMergeProjectTask(project, ctx, task, projectTask)
                    }
                ], { concurrent: false, exitOnError: true });
            }
        };
    });
    const tasks = new Listr(projTasks, {
        concurrent: config.maxConcurrent || hg_1.MAX_CONCURRENT_HGS,
        exitOnError: false
    });
    return tasks.run();
}
exports.merge = merge;
function tryMergeProjectTask(project, ctx, task, projectTask) {
    return rxjs_1.Observable.create((subscriber) => {
        checkIfNeedsMerge(project, ctx, task, projectTask)
            .subscribe((data) => subscriber.next(data), (err) => subscriber.error(err), () => {
            if (ctx.isSkipped) {
                delete ctx.isSkipped;
                subscriber.complete();
            }
            else {
                let projectTitleWithBranch = projectTask.title;
                projectTask.title = `${projectTitleWithBranch} (merging in ${project.defaultBranch})`;
                hg_1.mergeBranchIn(project.path, project.defaultBranch)
                    .subscribe((data) => subscriber.next(data), (err) => subscriber.error(err), () => {
                    hg_1.commit(project.path, `Merged in ${project.defaultBranch}`)
                        .subscribe((data) => subscriber.next(data), (err) => subscriber.error(err), () => {
                        projectTask.title = `${projectTitleWithBranch} (merged & committed)`;
                        subscriber.complete();
                    });
                });
            }
        });
    });
}
function checkIfNeedsMerge(project, ctx, _, projectTask) {
    return rxjs_1.Observable.create((subscriber) => {
        let branchName;
        hg_1.getBranch(project.path)
            .subscribe((data) => {
            branchName = data;
            subscriber.next(branchName);
        }, (err) => subscriber.error(err), () => {
            branchName = (branchName || '').trim();
            projectTask.title = `${project.path} - ${branchName}`;
            if (isExcludedBranch(project, branchName)) {
                ctx.isSkipped = true;
                ctx.isExcludedBranch = true;
                projectTask.skip(`...because current branch is ${branchName}`);
                subscriber.complete();
            }
            else {
                let status;
                hg_1.getStatus(project.path)
                    .subscribe((data) => {
                    status = data;
                    subscriber.next(status);
                }, (err) => subscriber.error(err), () => {
                    status = (status || '').trim();
                    if (status.length) {
                        ctx.isSkipped = true;
                        projectTask.skip(`...because there are uncommitted changes`);
                        projectTask.title = `${project.path} - ${branchName} (${status.length})`;
                        subscriber.complete();
                    }
                    else {
                        let branchNeedsMerging;
                        subscriber.next(`Checking if branch already has "${project.defaultBranch}" merged in...`);
                        hg_1.checkIfBranchNeedsOtherBranchMergingIn(project.path, branchName, project.defaultBranch)
                            .subscribe((data) => branchNeedsMerging = data, (err) => subscriber.error(err), () => {
                            if (!branchNeedsMerging) {
                                ctx.isSkipped = true;
                                projectTask.skip(`...because latest "${project.defaultBranch}" is already merged in`);
                            }
                            subscriber.complete();
                        });
                    }
                });
            }
        });
    });
    function isExcludedBranch(project, branchName) {
        return project.defaultBranch === branchName
            || branchName === config_file_1.DEFAULT_BRANCH_NAME; // todo second check maybes not necessary?
    }
}
//# sourceMappingURL=merge.js.map