"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Listr = require('listr');
const operators_1 = require("rxjs/operators");
const hg_1 = require("../hg");
function pull(config, options) {
    let projTasks = config.projects.map(project => {
        return {
            title: project.path,
            task: () => {
                if (options.tryUpdate) {
                    return new Listr([
                        {
                            title: 'Pulling changes',
                            task: () => hg_1.pullChanges(project.path)
                        },
                        {
                            title: `Ensuring no uncommitted changes`,
                            task: (ctx) => {
                                return hg_1.getStatus(project.path).pipe(operators_1.filter(Boolean), operators_1.tap(_ => {
                                    ctx.hasUncommittedChanges = true;
                                }));
                            }
                        },
                        {
                            title: `Updating`,
                            skip: (ctx) => ctx.hasUncommittedChanges,
                            task: () => hg_1.update(project.path)
                        }
                    ], { concurrent: false });
                }
                else {
                    return hg_1.pullChanges(project.path);
                }
            }
        };
    });
    const tasks = new Listr(projTasks, {
        concurrent: config.maxConcurrent || hg_1.MAX_CONCURRENT_HGS,
        exitOnError: false
    });
    return tasks.run();
}
exports.pull = pull;
//# sourceMappingURL=pull.js.map