"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Listr = require('listr');
const rxjs_1 = require("rxjs");
const hg_1 = require("../hg");
function status(config, options) {
    let projTasks = config.projects.map(project => {
        return {
            title: project.path,
            task: (ctx, task) => getProjectStatusTask(project, options, ctx, task)
        };
    });
    const tasks = new Listr(projTasks, {
        concurrent: config.maxConcurrent || hg_1.MAX_CONCURRENT_HGS,
        exitOnError: false
    });
    return tasks.run();
}
exports.status = status;
function getProjectStatusTask(project, options, _, task) {
    return rxjs_1.Observable.create((subscriber) => {
        let branchName;
        hg_1.getBranch(project.path)
            .subscribe((data) => {
            branchName = data;
            subscriber.next(data);
        }, (err) => subscriber.error(err), () => {
            task.title = `${project.path} - ${branchName}`;
            if (options.full) {
                let status;
                hg_1.getStatus(project.path)
                    .subscribe((data) => {
                    status = data;
                    subscriber.next(status);
                }, (err) => subscriber.error(err), () => {
                    status = (status || '').trim();
                    let statusLines = status.split('\n');
                    if (statusLines.length) {
                        task.title = `${project.path} - ${branchName} (${statusLines.length})`;
                    }
                    subscriber.complete();
                });
            }
            else {
                subscriber.complete();
            }
        });
    });
}
//# sourceMappingURL=status.js.map