"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const fs = require("fs");
const path = require("path");
const os = require("os");
const CONFIG_FILE_NAME = `.hgpp`;
exports.DEFAULT_BRANCH_NAME = 'default';
function getConfig() {
    return new Promise((resolve, reject) => {
        const fullConfigPath = getConfigFullPath();
        fs.exists(fullConfigPath, (exists) => {
            if (!exists) {
                console.log(`No hgpp config file found! Creating default config file at:\n`, fullConfigPath);
                const defaultConfigFile = {
                    projects: []
                };
                try {
                    fs_1.writeFileSync(fullConfigPath, JSON.stringify(defaultConfigFile, null, 4));
                }
                catch (e) {
                    reject(new Error(`Unable to write a config file at ${fullConfigPath}.` + e.toString()));
                    return;
                }
            }
            fs.readFile(fullConfigPath, (err, buffer) => {
                if (err) {
                    reject(new Error(`Error reading .hgpp config file from ${fullConfigPath}. Error: ${err}`));
                    return;
                }
                let config;
                try {
                    let configFileText = buffer.toString();
                    config = JSON.parse(configFileText);
                }
                catch (exception) {
                    reject(new Error(`Error parsing .hgpp config file. Error: ${err}`));
                    return;
                }
                config.projects = config.projects.map(proj => {
                    proj.defaultBranch = proj.defaultBranch || exports.DEFAULT_BRANCH_NAME;
                    return proj;
                });
                resolve(config);
            });
        });
    });
}
exports.getConfig = getConfig;
function getConfigFullPath() {
    return path.resolve(os.homedir(), CONFIG_FILE_NAME);
}
exports.getConfigFullPath = getConfigFullPath;
//# sourceMappingURL=config-file.js.map